@@ -64,11 +64,12 @@ typedef struct {
 
 static pilot_properties properties = { NULL };
 
-typedef enum { INITIALIZING,SYNCING,WAITING, NUM_STATES } state;
+typedef enum { INITIALISING, CONNECTING_TO_DAEMON, SYNCING, WAITING, NUM_STATES } state;
 
 char *pixmaps[] = 
 {
     "sync_broken.png",
+    "sync_icon.png",
     "syncing_icon.png",
     "sync_icon.png",
 };
@@ -398,12 +399,12 @@ timeout (gpointer data)
 {
 	switch (curstate) {
 		
-	case INITIALIZING: {
-		g_message ("state = INITIALIZING");
+	case INITIALISING: {
+		g_message ("state = INITIALISING");
 		pilot_draw (GTK_WIDGET (data));
 		
 		if (gnome_pilot_client_connect_to_daemon (gpc) == GPILOTD_OK) {
-			curstate = WAITING;
+			curstate = CONNECTING_TO_DAEMON;
 			pilot_draw (GTK_WIDGET (data));
 		} else {
 			/* timeout_handler_id = gtk_timeout_add (3000,timeout,data); */
@@ -411,9 +412,11 @@ timeout (gpointer data)
 		return TRUE;
 	} break;
 	
-	case WAITING: {
-		g_message ("state = WAITING");
-                gtk_timeout_remove (timeout_handler_id);
+	case CONNECTING_TO_DAEMON: {
+		g_message ("state = CONNECTING_TO_DAEMON");
+
+                gtk_timeout_remove (timeout_handler_id); 
+		timeout_handler_id = gtk_timeout_add (5000,timeout,pixmap);
 		
 		if (properties.pilot_ids) {
 			GList *tmp =properties.pilot_ids;
@@ -427,7 +430,7 @@ timeout (gpointer data)
 		if (properties.pilot_ids==NULL){
 			applet_widget_set_tooltip (APPLET_WIDGET (applet),_("Not connected. Restart daemon to reconnect"));
 			pilot_execute_program (GPILOTD_DRUID);
-			curstate=INITIALIZING;
+			curstate=INITIALISING;
 			pilot_draw (GTK_WIDGET (data));
 			/* FIXME: add gpilot_monitor_state_change () */
 			return FALSE;
@@ -440,9 +443,18 @@ timeout (gpointer data)
 		
 		applet_widget_set_tooltip (APPLET_WIDGET (applet),_("Ready to synchronize"));
 		
+		curstate = WAITING;
 		pilot_draw (GTK_WIDGET (data));    
 	} break;
-	case SYNCING: break;
+	case WAITING:
+		if (gnome_pilot_client_noop (gpc) == GPILOTD_ERR_NOT_CONNECTED) {
+			curstate = INITIALISING;
+			gtk_timeout_remove (timeout_handler_id); 
+			timeout_handler_id = gtk_timeout_add (1000,timeout,pixmap);
+		}
+		break;
+	case SYNCING: 
+	break;
 	default: g_error ("curstate == default ?!"); break;
 	}
 	
@@ -501,7 +513,7 @@ handle_client_error (void)
 	if (curstate == SYNCING) {
 		show_warning_dialog (_("Pilot is currently synchronizing.\nPlease wait for it to finish."));
 	} else {
-		curstate=INITIALIZING;
+		curstate=INITIALISING;
 		applet_widget_set_tooltip (APPLET_WIDGET (applet),_("Not connected. Please restart daemon."));	
 		pilot_draw (pixmap);
 		show_error_dialog (_("Not connected to gpilotd.\nPlease restart daemon."));
@@ -961,7 +973,7 @@ restart_cb (AppletWidget *widget, gpointer data)
 			gtk_widget_hide (progressDialog);
 		}
 	}
-	curstate = INITIALIZING;
+	curstate = INITIALISING;
 	gnome_pilot_client_restart_daemon (gpc);
 	applet_widget_set_tooltip (APPLET_WIDGET (applet),_("Trying to connect to "
 							  "the GnomePilot Daemon"));
@@ -1020,7 +1032,7 @@ create_pilot_widget (GtkWidget *window)
 							  "the GnomePilot Daemon"));
 
 
-	curstate = INITIALIZING;
+	curstate = INITIALISING;
 
 	for (i = 0; i < sizeof (pixmaps)/sizeof (pixmaps[0]); i++)
 		pixmaps[i] = gnome_unconditional_pixmap_file (pixmaps[i]);
