@@ -1,6 +1,35 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- *//* 
+ * Copyright (C) 1998-2000 Free Software Foundation
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Eskil Heyn Olsen
+ *
+ */
+
 #include "config.h"
 #include <gnome.h>
+
+#ifdef USING_OAF
+#include <liboaf/liboaf.h>
+#include <bonobo.h>
+#else
 #include <libgnorba/gnorba.h>
+#endif
+
 #include "orbit_daemon_glue.h"
 #include "queue_io.h"
 #include "manager.h"
@@ -98,10 +127,13 @@ enum ForEachAction {
 	CALL_CONNECT,
 	CALL_DISCONNECT,
 	CALL_CONDUIT_PROGRESS,
+	CALL_OVERALL_PROGRESS,
 	CALL_CONDUIT_MESSAGE,
 	CALL_CONDUIT_ERROR,
 	CALL_CONDUIT_BEGIN,
-	CALL_CONDUIT_END
+	CALL_CONDUIT_END,
+	CALL_DAEMON_MESSAGE,
+	CALL_DAEMON_ERROR
 };
 
 /* This is used to pass arguments to various _foreach calls */
@@ -207,8 +239,10 @@ get_client_id(const GNOME_Pilot_Client cb)
 static void
 pause_device(GPilotDevice *device,gpointer data)
 {
-	g_source_remove(device->in_handle);
-	g_source_remove(device->err_handle);
+	if (device->io) {
+		g_source_remove(device->in_handle);
+		g_source_remove(device->err_handle);
+	}
 }
 
 /********************************************************************************/
@@ -234,6 +268,13 @@ gpilotd_corba_pause(GNOME_Pilot_Daemon obj,
 	return;
 }
 
+static void
+gpilotd_corba_noop (GNOME_Pilot_Daemon obj,
+		    CORBA_Environment *ev)
+{
+	return;
+}
+
 static CORBA_unsigned_long
 gpilotd_corba_request_install(GNOME_Pilot_Daemon obj,
 			      const GNOME_Pilot_Client cb,
@@ -436,7 +477,7 @@ gpilotd_corba_get_user_info(GNOME_Pilot_Daemon obj,
 static CORBA_unsigned_long
 gpilotd_corba_get_system_info(GNOME_Pilot_Daemon obj,
 			      const GNOME_Pilot_Client cb,
-			      const CORBA_char *pilot_name, 
+			      const CORBA_char *cradle, 
 			      const GNOME_Pilot_Survival survival,
 			      const CORBA_unsigned_long timeout,
 			      CORBA_Environment *ev) 
@@ -445,17 +486,13 @@ gpilotd_corba_get_system_info(GNOME_Pilot_Daemon obj,
 	gchar *client_id;
 	GET_AND_CHECK_CLIENT_ID(client_id,cb,0);
 	
-	LOG("corba: get_system_info(pilot=%s,survival=%d,timeout=%d)",
-	    pilot_name,survival,timeout);
+	LOG("corba: get_system_info(cradle=%s,survival=%d,timeout=%d)",
+	    cradle ,survival,timeout);
 	
 	req.type = GREQ_GET_SYSINFO;
-	req.pilot_id = pilot_id_from_name(pilot_name,orb_context->gpilotd_context);
-	if(req.pilot_id==0) {
-		g_free(client_id);
-		gpthrow_with_settings(UnknownPilot,0,exn,exn->pilotId = CORBA_string_dup(pilot_name));
-	}
+	req.pilot_id = 0;
 	req.timeout = survival==GNOME_Pilot_PERSISTENT?0:timeout;
-	req.cradle = NULL;
+	req.cradle = g_strdup (cradle);
 	req.client_id = g_strdup(client_id);
 	
 	g_free(client_id);
@@ -468,6 +505,7 @@ gpilotd_corba_set_user_info(GNOME_Pilot_Daemon obj,
 			    const GNOME_Pilot_Client cb,
 			    GNOME_Pilot_PilotUser *user,
 			    const CORBA_char *cradle, 
+			    const CORBA_boolean continue_sync,
 			    const GNOME_Pilot_Survival survival,
 			    const CORBA_unsigned_long timeout, 
 			    CORBA_Environment *ev) 
@@ -492,6 +530,7 @@ gpilotd_corba_set_user_info(GNOME_Pilot_Daemon obj,
 	req.parameters.set_userinfo.password = NULL;
 	req.parameters.set_userinfo.user_id = g_strdup(user->username);
 	req.parameters.set_userinfo.pilot_id = user->userID;
+	req.parameters.set_userinfo.continue_sync = continue_sync;
 
 	g_free(client_id);
 
@@ -1074,11 +1113,12 @@ gpilotd_corba_get_pilots_by_login(GNOME_Pilot_Daemon obj,
 	setpwent();
 	do {
 		pwdent = getpwent();
-		if(pwdent) 
+		if(pwdent) {
 			if(strcmp(pwdent->pw_name,uid)==0) {
 				username = strdup(pwdent->pw_gecos);
 				pwdent=NULL; /* end the loop */
 			}
+		}
 	} while(pwdent);
 	endpwent();
 
@@ -1100,10 +1140,13 @@ gpilotd_corba_get_user_name_by_pilot(GNOME_Pilot_Daemon obj,
 				     const CORBA_char * pilot_name, 
 				     CORBA_Environment *ev) 
 {
-	LOG("corba: FIXME get_user_name_by_pilot");
+#ifdef __GNUC__
+#warning gpilotd_corba_get_user_name_by_pilot not implemented
+#endif
+	LOG("corba: FIXME %s:%d get_user_name_by_pilot", __FILE__, __LINE__);
 
 	/* FIXME: not implemented yet */
-	return NULL;
+	return CORBA_string_dup ("Foo Bar");
 }
 
 static CORBA_char*
@@ -1111,10 +1154,13 @@ gpilotd_corba_get_user_login_by_pilot(GNOME_Pilot_Daemon obj,
 				      const CORBA_char * pilot_name, 
 				      CORBA_Environment *ev) 
 {
-	LOG("corba: FIXME get_user_login_by_pilot");
+#ifdef __GNUC__
+#warning gpilotd_corba_get_user_login_by_pilot not implemented
+#endif
+	LOG("corba: FIXME %s:%d get_user_login_by_pilot", __FILE__, __LINE__);
 
 	/* FIXME: not implemented yet */
-	return NULL;
+	return CORBA_string_dup ("foo");
 }
 
 static CORBA_char* 
@@ -1122,7 +1168,7 @@ gpilotd_get_pilot_base_dir(PortableServer_Servant _servant,
 		     CORBA_char * pilot_name, 
 		     CORBA_Environment * ev) {
 	int i;
-	LOG("corba: get_pilot_base_dir(id=%s:",pilot_name);
+	LOG("corba: get_pilot_base_dir(id=%s)",pilot_name);
 	
 
 	/* FIXME: this could probably be prettyfied with some _foreach thingies... */
@@ -1280,6 +1326,7 @@ PortableServer_ServantBase__epv base_epv = {
 POA_GNOME_Pilot_Daemon__epv daemon_epv = { 
 	NULL, 
 	(gpointer)&gpilotd_corba_pause,
+	(gpointer)&gpilotd_corba_noop,
 	(gpointer)&gpilotd_corba_request_install,
 	(gpointer)&gpilotd_corba_request_restore,
 	(gpointer)&gpilotd_corba_request_conduit,
@@ -1374,6 +1421,17 @@ orbed_notify_multi_purpose_foreach(gchar *IOR,
 							   carrier->ptr1, 
 							   carrier->ptr2, &ev);
 			break;
+		case CALL_DAEMON_MESSAGE:
+			GNOME_Pilot_Client_daemon_message(cb, 
+							  carrier->id, 
+							  carrier->ptr1 ? carrier->ptr1 : "", 
+							  carrier->ptr2, &ev);
+			break;
+		case CALL_DAEMON_ERROR:
+			GNOME_Pilot_Client_daemon_error (cb, 
+							 carrier->id, 
+							 carrier->ptr2, &ev);
+			break;
 		case CALL_CONDUIT_ERROR:
 			GNOME_Pilot_Client_conduit_error(cb, 
 							 carrier->id, 
@@ -1387,6 +1445,12 @@ orbed_notify_multi_purpose_foreach(gchar *IOR,
 							    carrier->long1,
 							    carrier->long2, &ev);
 			break;
+		case CALL_OVERALL_PROGRESS:
+			GNOME_Pilot_Client_overall_progress(cb, 
+							    carrier->id, 
+							    carrier->long1,
+							    carrier->long2, &ev);
+			break;
 		case CALL_CONDUIT_BEGIN:
 			GNOME_Pilot_Client_conduit_start(cb, 
 							 carrier->id, 
@@ -1590,7 +1654,8 @@ orbed_notify_userinfo(struct PilotUser user_info,
                                                                                 */
 void 
 orbed_notify_sysinfo(gchar *pilot_id,
-		     struct CardInfo sysinfo,
+		     struct SysInfo sysinfo,
+		     struct CardInfo cardinfo,
 		     GPilotRequest **req) 
 {
 	GNOME_Pilot_SysInfo _sysinfo;
@@ -1605,12 +1670,21 @@ orbed_notify_sysinfo(gchar *pilot_id,
 		CORBA_exception_free(&ev);
 		return;
 	}
-
-	_sysinfo.romSize = sysinfo.romSize;
-	_sysinfo.ramSize = sysinfo.ramSize;
-	_sysinfo.ramFree = sysinfo.ramFree;
-	_sysinfo.name = g_strdup(sysinfo.name);
-	_sysinfo.manufacturer = g_strdup(sysinfo.manufacturer);
+/*
+	g_message ("sysinfo.romVersion = %lu", sysinfo.romVersion);
+	g_message ("sysinfo.name = %s", sysinfo.name);
+	g_message ("cardinfo.version = %d", cardinfo.version);
+	g_message ("cardinfo.creation = %lu", cardinfo.creation);
+	g_message ("cardinfo.name = %s", cardinfo.name);
+	g_message ("cardinfo.manufacturer = %s", cardinfo.manufacturer);
+*/
+	_sysinfo.romSize = cardinfo.romSize/1024;
+	_sysinfo.ramSize = cardinfo.ramSize/1024;
+	_sysinfo.ramFree = cardinfo.ramFree/1024;
+	_sysinfo.name = g_strdup(cardinfo.name);
+	_sysinfo.manufacturer = g_strdup(cardinfo.manufacturer);
+	_sysinfo.creation = cardinfo.creation;
+	_sysinfo.romVersion = sysinfo.romVersion;
 
 	GNOME_Pilot_Client_sysinfo_requested(cb, pilot_id,&_sysinfo,&ev);
 	if(ev._major != CORBA_NO_EXCEPTION) {
@@ -1646,7 +1720,7 @@ orbed_notify_conduit_start(gchar *pilot_id,
 	}
 
 	carrier.id = g_strdup(pilot_id);
-	carrier.ptr1 = g_strdup(gnome_pilot_conduit_get_name(conduit));
+	carrier.ptr1 = gnome_pilot_conduit_get_name(conduit);
 	if(GNOME_IS_PILOT_CONDUIT_STANDARD(conduit))
 		carrier.ptr2 = g_strdup(gnome_pilot_conduit_standard_get_db_name(GNOME_PILOT_CONDUIT_STANDARD(conduit)));
 	else
@@ -1677,7 +1751,7 @@ orbed_notify_conduit_end(gchar *pilot_id,
 	}
 
 	carrier.id = g_strdup(pilot_id);
-	carrier.ptr1 = g_strdup(gnome_pilot_conduit_get_name(conduit));
+	carrier.ptr1 = gnome_pilot_conduit_get_name(conduit);
 	carrier.purge_list = NULL;
 	carrier.action = CALL_CONDUIT_END;
 	g_slist_foreach(notifications->conduit,
@@ -1704,7 +1778,7 @@ orbed_notify_conduit_error(gchar *pilot_id,
 	}
 
 	carrier.id = g_strdup(pilot_id);
-	carrier.ptr1 = g_strdup(gnome_pilot_conduit_get_name(conduit));
+	carrier.ptr1 = gnome_pilot_conduit_get_name(conduit);
 	carrier.ptr2 = message;
 	carrier.purge_list = NULL;
 	carrier.action = CALL_CONDUIT_ERROR;
@@ -1732,7 +1806,7 @@ orbed_notify_conduit_message(gchar *pilot_id,
 	}
 
 	carrier.id = g_strdup(pilot_id);
-	carrier.ptr1 = g_strdup(gnome_pilot_conduit_get_name(conduit));
+	carrier.ptr1 = gnome_pilot_conduit_get_name(conduit);
 	carrier.ptr2 = message;
 	carrier.purge_list = NULL;
 	carrier.action = CALL_CONDUIT_MESSAGE;
@@ -1745,6 +1819,54 @@ orbed_notify_conduit_message(gchar *pilot_id,
 	g_free(carrier.ptr1);
 }
 
+void 
+orbed_notify_daemon_message(gchar *pilot_id,				
+			    GnomePilotConduit *conduit,
+			    gchar *message) {
+	GPilotd_notify_foreach_carrier carrier;
+	GPilotd_Orb_Pilot_Notifications *notifications;
+	
+	if((notifications = g_hash_table_lookup(orb_context->notifications,pilot_id))==NULL) {
+		return;
+	}
+
+	carrier.id = g_strdup(pilot_id);
+	carrier.ptr1 = conduit ? gnome_pilot_conduit_get_name(conduit) : NULL;
+	carrier.ptr2 = message;
+	carrier.purge_list = NULL;
+	carrier.action = CALL_DAEMON_MESSAGE;
+	g_slist_foreach(notifications->conduit,
+			(GFunc)orbed_notify_multi_purpose_foreach,
+			&carrier);
+	/* FIXME: See note 1 at the end of this file */
+	if(carrier.purge_list) purge_ior_foreach(pilot_id,carrier.purge_list);
+	g_free(carrier.id);
+	g_free(carrier.ptr1);
+}
+
+void 
+orbed_notify_daemon_error (gchar *pilot_id,				
+			   gchar *message) {
+	GPilotd_notify_foreach_carrier carrier;
+	GPilotd_Orb_Pilot_Notifications *notifications;
+	
+	if((notifications = g_hash_table_lookup(orb_context->notifications,pilot_id))==NULL) {
+		return;
+	}
+
+	carrier.id = g_strdup(pilot_id);
+	carrier.ptr2 = message;
+	carrier.purge_list = NULL;
+	carrier.action = CALL_DAEMON_ERROR;
+	g_slist_foreach(notifications->conduit,
+			(GFunc)orbed_notify_multi_purpose_foreach,
+			&carrier);
+	/* FIXME: See note 1 at the end of this file */
+	if(carrier.purge_list) purge_ior_foreach(pilot_id,carrier.purge_list);
+	g_free(carrier.id);
+	g_free(carrier.ptr1);
+}
+
 void 
 orbed_notify_conduit_progress(gchar *pilot_id,				
 			      GnomePilotConduit *conduit,
@@ -1758,10 +1880,9 @@ orbed_notify_conduit_progress(gchar *pilot_id,
 	}
 
 	carrier.id = g_strdup(pilot_id);
-	carrier.ptr1 = g_strdup(gnome_pilot_conduit_get_name(conduit));
+	carrier.ptr1 = gnome_pilot_conduit_get_name(conduit);
 	carrier.long1 = current;
 	carrier.long2 = total;
-	g_message("orbed_notify_conduit_progress(cur=%ld,tot=%ld)",carrier.long1,carrier.long2);
 	carrier.purge_list = NULL;
 	carrier.action = CALL_CONDUIT_PROGRESS;
 	g_slist_foreach(notifications->conduit,
@@ -1773,6 +1894,30 @@ orbed_notify_conduit_progress(gchar *pilot_id,
 	g_free(carrier.ptr1);
 }
 
+void 
+orbed_notify_overall_progress(gchar *pilot_id,				
+			      int current, 
+			      int total) {
+	GPilotd_notify_foreach_carrier carrier;
+	GPilotd_Orb_Pilot_Notifications *notifications;
+	
+	if((notifications = g_hash_table_lookup(orb_context->notifications,pilot_id))==NULL) {
+		return;
+	}
+
+	carrier.id = g_strdup(pilot_id);
+	carrier.long1 = current;
+	carrier.long2 = total;
+	carrier.purge_list = NULL;
+	carrier.action = CALL_OVERALL_PROGRESS;
+	g_slist_foreach(notifications->conduit,
+			(GFunc)orbed_notify_multi_purpose_foreach,
+			&carrier);
+	/* FIXME: See note 1 at the end of this file */
+	if(carrier.purge_list) purge_ior_foreach(pilot_id,carrier.purge_list);
+	g_free(carrier.id);
+}
+
 /********************************************************************************/
 
 void 
@@ -1858,33 +2003,49 @@ gpilotd_corba_quit(void)
 	CORBA_ORB_shutdown(orb, CORBA_FALSE, &ev);
 }
 
+#ifdef USING_OAF
+static ORBit_MessageValidationResult
+accept_all_cookies (CORBA_unsigned_long request_id,
+		    CORBA_Principal *principal,
+		    CORBA_char *operation)
+{
+        /* allow ALL cookies */
+	return ORBIT_MESSAGE_ALLOW_ALL;
+}
+#endif
+
 void gpilotd_corba_init(int *argc, 
 			char **argv,
-			GPilotContext **context) 
+			GPilotContext **context)
 {
 	PortableServer_ObjectId objid = {0, sizeof("gpilotd"), "gpilotd" };
 	PortableServer_POA the_poa;
 	GNOME_Pilot_Daemon acc;
 	static gboolean object_initialised = FALSE;
-
+#ifdef USING_OAF
+	OAF_RegistrationResult result;
+#endif
 	if ( object_initialised ) return;
 
 	object_initialised = TRUE;
 
 	CORBA_exception_init(&ev);
 
+#ifdef USING_OAF
+	orb = oaf_init (*argc, argv);
+	ORBit_set_request_validation_handler (accept_all_cookies);
+#else
 	orb = gnome_CORBA_init(PACKAGE,VERSION,
 			       argc,argv,
 			       GNORBA_INIT_SERVER_FUNC,&ev);
 	g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);
+#endif
 
 	*context = gpilot_context_new();
 	gpilot_context_init_user (*context);
 	gpilotd_corba_init_context(*context);
 
-	the_poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, 
-									   "RootPOA", 
-									   &ev);
+	the_poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
 	g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);
 
 	POA_GNOME_Pilot_Daemon__init(&daemon_servant, &ev);
@@ -1899,27 +2060,58 @@ void gpilotd_corba_init(int *argc,
 	acc = PortableServer_POA_servant_to_reference(the_poa, &daemon_servant, &ev);
 	g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);
 
+	g_message ("Activating server");
+#ifdef USING_OAF
+	result = oaf_active_server_register (
+              "OAFIID:gpilotd:a40c417b-21db-4959-a58e-40096a0daf09",
+	      CORBA_OBJECT (acc));
+	g_message ("oaf_active_server_register = %d", result);
+	switch (result) {
+	case OAF_REG_SUCCESS:
+		break;
+	case OAF_REG_NOT_LISTED:
+		g_message ("Cannot register gpilotd because not listed");
+		exit (1);
+	case OAF_REG_ALREADY_ACTIVE:
+		g_message ("Cannot register gpilotd because already active");
+		exit (0);
+	case OAF_REG_ERROR:
+	default:
+		g_message ("Cannot register gpilotd because we suck");
+		exit (1);
+	}
+#else 
 	switch(goad_server_register(CORBA_OBJECT_NIL,acc,"gpilotd","object",&ev)) {
 	case -2:
-		/* FIXME: better handling here */
+                /* FIXME: better handling here */
 		g_message(_("The gnome-pilot daemon is already running"));
 		exit (0);
 		break;
 	case -1:
-		/* FIXME: better handling here */
+                /* FIXME: better handling here */
 		g_print("GOAD Name server failure\n");
 		exit (1);
 		break;
 	}
 	g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);
+#endif	
 
 	CORBA_Object_release(acc, &ev);
 	g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);
 
 	ORBit_custom_run_setup(orb, &ev);
 	g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);
+
+#ifdef USING_OAF
+	if (!bonobo_init (CORBA_OBJECT_NIL, CORBA_OBJECT_NIL,
+			  CORBA_OBJECT_NIL)) {
+		g_message ("init_bonobo(): could not initialize Bonobo");
+		exit (EXIT_FAILURE);
+	}
+#endif
 }
 
+
 /* Notes 
    
 Note 1:
